YII-SORTER ([English version](https://github.com/wartur/yii-sorter/blob/master/README.md))
==========================================================================================

Расширение для Yii для работы с упорядоченным списком управляемым пользователем. При использовании данного расширения не забывайте пользоваться транзакциями или блокировками таблиц.

ДЕМО: http://yii-sorter.wartur.ru

[Описание алгоритма](https://github.com/wartur/yii-sorter/blob/master/ALGORITHM.ru.md)

## Релиз 1.0.0 для Yii первый и последний. Дальнейшая разработка [будет проводиться на Yii2](https://github.com/wartur/yii2-sorter)

###### От автора
> Данное расширение создавалось для фана и для удовлетворения своего
> низменного, пошлого и никому не нужного в практике перфекционизма,
> я отдаю себе отчет в том, что данную задачу можно решить менее оптимально
> и заметно менее трудоемко. Я стремился создать самый удобный, быстрый
> и стабильный механизм для решения достаточно простой и достаточно частой
> в своей практике задачи

Абстракт
--------
Расширение позволяет работать с упорядоченным списком, формируемым пользователем.
Расширение обеспечивает высочайшую скорость работы с базой данных. Для вставки
записи в произвольное места упорядоченного списка требуется один запрос на запись
в базу данных.

Примеры использования: порядок вывода статей на страницу, расположение виджетов на экране. Благодаря
использованному алгоритму, данное расширение удобно использовать для работы с
очень большими массивами данных, которые невозможно уместить в оперативную память.

Расширение оформлено как компонент приложения, поведения для CActiveRecord и набора готовых
виджетов и действий для работы с ним.

Компонент используется для поддержки функционала подсветки записей как звено между
отображением и CActiveRecord

Работа с записями происходит в полном соответствии с концепцией ActiveRecord.
Расширение не использует прямых обращений в базу на запись. Так что можно не волноваться
за исполнение beforeSave, afterSave. Все ваши события над записями отработают без ошибок, включая те, которые были изменены косвенно.

Алгоритм протестирован при настройках по умолчанию. Он протестирован в некоторых
экстремальных условиях и при настройках отличных от настроек по умолчанию.
Покрытие кода более 98% (на самом деле 92% - 6% не покрывают некритичный участок кода,
который требуется только при разработки проекта и помогает разработчику).
За целостность данных можно не волноваться ПРИ СОБЛЮДЕНИИ работы через
транзакции/блокировки или при использовании действий данного
расширения.

ВАЖНО: все операции с использованием данного расширения ДОЛЖНЫ производиться через
транзакции(ISOLATION LEVEL SERIALIZABLE)/блокировки. В противном случае
есть вероятность разрушения базы данных.
Все действия данного расширения включают в себя транзакции, в случаях когда таблица
не поддерживает транзакции в компоненте требуется указать,
что требуется использовать блокировку таблицы.

Подключение расширения к проекту
--------------------------------
1) [Скачайте новейший релиз](https://github.com/wartur/yii-sorter/releases)

2) Распакуйте yii-sorter в директории ext.wartur.yii-sorter

3) Добавьте новый алиас пути в начало конфигурационного файла (по умолчанию: config/main.php)
```php
Yii::setPathOfAlias('sorter', 'protected/extensions/wartur/yii-sorter');
```

4) Добавьте новый компонент приложения в конфигурационный файл.
Минимальная конфигурация:
```php
'components'=>array(
	'sorter' => array(
		'class' => 'sorter.components.Sorter',
	),
	// ....
)
```

5) Добавьте поведение в модель в которой требуется использовать упорядоченный список.
Минимальная конфигурация:
```php
public function behaviors() {
	return array_merge(parent::behaviors(), array(
		'SorterActiveRecordBehavior' => array(
			'class' => 'sorter.behaviors.SorterActiveRecordBehavior',
		)
	));
}
```

6) Проверьте, что ваша модель удовлетворяет схеме, указанной в [sorter.tests.env.schema](https://github.com/wartur/yii-sorter/blob/master/tests/env/schema/sortest.sql).
Помните для работы поведения требуется поле SIGNED INT sort с УНИКАЛЬНЫМ ключом. Подробнее
вы можете прочитать в [API reference поведения](https://github.com/wartur/yii-sorter/blob/master/behaviors/SorterActiveRecordBehavior.php)

Работа с готовыми виджетами
---------------------------
К расширению поставляется базовый набор виджетов.
Используя текущее API, вы можете написать свой виджет, остальное за вас
сделает поведение. Вам не нужно знать, как оно сработает на низком уровне, важно
понимать, что оно сработает очень быстро и правильно.

1) Добавьте к таблице CGridView колонку с простым интерфейсом управления позициями.
Минимальная конфигурация:
```php
<?php
$this->widget('zii.widgets.grid.CGridView', array(
	'dataProvider' => $model->search(),
	'columns' => array(
		'id',
		array(
			'class' => 'sorter.widgets.SorterButtonColumn',
		),
	)
));
?>
```

2) Добавьте к таблице колонку для перемещения текущей строки перед определенной позицией.
Минимальная конфигурация:
```php
<?php
$this->widget('zii.widgets.grid.CGridView', array(
	'dataProvider' => $model->search(),
	'columns' => array(
		'id',
		array(
			'class' => 'sorter.widgets.SorterDropDownColumn',
		),
	)
));
?>
```

3) Для работы виджетов с контроллером, вам требуется добавить в контроллер
действия и поведение. Поведение обеспечивает связь контроллера с моделью.
Полный набор действий являются полной имплиментацией API поведения
```php
public function actions() {
	return array_merge(parent::actions(), array(
		'sorterMoveNumber' => 'sorter.actions.SorterMoveNumberAction',
		'sorterMoveOne' => 'sorter.actions.SorterMoveOneAction',
		'sorterMoveToEdge' => 'sorter.actions.SorterMoveToEdgeAction',
		'sorterMoveToModel' => 'sorter.actions.SorterMoveToModelAction',
		'sorterMoveToPosition' => 'sorter.actions.SorterMoveToPositionAction',
	));
}

public function behaviors() {
	return array_merge(parent::behaviors(), array(
		'SorterControllerBehavior' => array(
			'class' => 'sorter.behaviors.SorterControllerBehavior',
			'className' => 'Sortest',
		),
	));
}
```

Подробнее об этих виджетах вы можете прочитать в API reference
в директории [sorter.widgets](https://github.com/wartur/yii-sorter/tree/master/widgets),
у них есть дополнительные параметры настройки

Работа с подсветкой-вспышкой
----------------------------
Для подключения подсветки требуется в конфигурационном файле
в конфигурации компонента добавить параметр useFlashHighligh.
Данная настройка указывает компоненту на то, что требуется сохранять данные
об операциях над моделями для последующего принятия решения о подсветке
```php
'components'=>array(
	'sorter' => array(
		'class' => 'sorter.components.Sorter',
		'useFlashHighligh' => true
	),
	// ....
)
```

Подсветка выполнена в качестве методов хелперов для настройки CGvidView.
Минимальная конфигурация:
```php
<?php
$this->widget('zii.widgets.grid.CGridView', array(
	'dataProvider' => $model->search(),
	'rowHtmlOptionsExpression' => 'Yii::app()->sorter->fhGridRowHtmlOptionsExpression($data)',
	'afterAjaxUpdate' => Yii::app()->sorter->fhGridAfterUpdateCode(true),
	'columns' => array(
		'id',
		array(
			'class' => 'sorter.widgets.SorterButtonColumn',
		),
	)
));
?>
```

У хелперов Sorter::fhGridRowHtmlOptionsExpression и Sorter::fhGridAfterUpdateCode
есть дополнительные параметры кастомизации, подробнее в [API reference](https://github.com/wartur/yii-sorter/blob/master/components/Sorter.php)
данных методов

Используя дополнительные настройки хелперов компонента Sorter, вы сможете вместо вспышки
использовать выделение selected встроенное в yii. Для этого надо поменять
используемый параметр Sorter::$flashHighlightCssClass = 'selected'. В этом случае
afterAjaxUpdate задавать не требуется.

Обеспечение целостности на таблицах, не поддерживающие транзакции
-----------------------------------------------------------------
Действия имплементированы с использованием транзакций с максимальной изоляцией.
В некоторых случаях транзакцию выполнить невозможно. Из-за этого приходится
пользоваться блокированием таблицы целиком. В готовых действиях уже все имплементировано,
вам требуется только добавить в конфигурационном файле в конфигурацию компонента
соответствующую настройку:
```php
'components'=>array(
	'sorter' => array(
		'class' => 'sorter.components.Sorter',
		'useLockTable' => true
	),
	// ....
)
```

Отладочная информация. Для фанатов подкапотного пространства
---------------------
Можно добавить данную колонку и посмотреть как перемещаются биты или изменяюся натуральные величины поля sort =)
```php
'columns' => array(
	array(
		'type' => 'raw',
		'name' => 'sort',
		'value' => '"(".str_pad(decbin($data->sort), 32, "0", STR_PAD_LEFT).")<br>".$data->sort'
	),
	// ...
)
```

Удачной работы!
